
// Begin include libraries
#include <BobShield.h> // Custom library for Bob Shield v3
#include <Commands.h>
#include <GPGGA.h>

/* In "/libraries/Wire/utility/twi.h" make sure TWI_FREQ (line 28) is defined as 400000L */
#include <Wire.h> // I2C communications (supporting the LSM303 library)
#include <LSM303.h> // Library for the compass board
#include <L3G.h> // Library for the L3GD20 gyro
#include <LPS331.h> // Library for a pressure sensor

#include <Flash.h> // Library for the NAND Flash
#include <SPI.h> // SPI communications (supporting the NAND-Flash library)
#include <EEPROM.h> // For non-volatile memory on the Arduino (supporting the NAND-Flash library)
// End include libraries

#define SAMPLE_INTERVAL 10000// 100 Hz
#define GPS_INTERVAL 100000 // 10 Hz
#define dnt_rate 3;
#define MAX_GPS_DATA 100 // 100 bytes
#define PRESS A0
#define TEMP A1
#define nandReset 0

byte dnt_count;

Flash* nandFlash; // Pointer to nandflash object
BobShield* bobShield; // Pointer to BobShield object
LSM303 compass; // Compass object
L3G gyro; // Gyro object
LPS331 ps; // Pressure sensor object

// At the start of each line of data
byte GPS_SENTINEL = '$'; // To indicate GPS data
byte DATA_SENTINEL = '#'; // To indicate a line of compass data

void setup() {

  Serial.begin(115200);
  nandFlash = new Flash(true, nandReset); // Flash(enable_write, restart_address_counter)
  bobShield = new BobShield();

  // Compass setup
  Wire.begin();
  compass.init(); //Turns on LSM303DLH
  compass.writeAccReg(LSM303_CTRL_REG1_A, 0x57); // Acc output data rate (ODR) = 100 Hz, not low power 
  compass.writeAccReg(LSM303_CTRL_REG4_A, 0x18); // Set accelerometer scale to +/-4G w HR
  compass.writeMagReg(LSM303_MR_REG_M, 0x00);    // Default
  compass.writeMagReg(LSM303_CRA_REG_M, 0x18);   // Mag output data rate (ODR) = 75 Hz
  compass.writeMagReg(LSM303_CRB_REG_M, 0x20);   // Mag gain +/-1.3 Gauss (X,Y - 1100 LSB/Gauss, Z - 980 LSB/Gauss)

  gyro.init(); //Turns on L3GD20
  gyro.writeReg(L3G_CTRL_REG1, 0x0F); // ODR 95Hz, high-pass filter cutoff 12.5, normal mode, all axes
  gyro.writeReg(L3G_CTRL_REG2, 0x00); // Normal mode hp-filter, 7.2 Hz cutoff @ 95Hz ODR
  gyro.writeReg(L3G_CTRL_REG4, 0x30); // Continuous update, little endian, scale 2000 dps

  ps.init(); //Turns on LPS331
  ps.enableDefault();
//  ps.writeReg(LPS331_CTRL_REG1, 0x70); // ODR 25 Hz (decreases resolution)

  // adjust the prescale for the analog inputs (http://arduino.cc/forum/index.php/topic,6549.0.html)
  // defines for setting and clearing register bit
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

  // set prescale to 32
  sbi(ADCSRA,ADPS2) ;
  cbi(ADCSRA,ADPS1) ;
  sbi(ADCSRA,ADPS0) ;

  pinMode(PRESS,INPUT);
  pinMode(TEMP,INPUT);
  
  bobShield->dntReset();
}

void loop() {
  // GPS data
  byte GPSdata[MAX_GPS_DATA]; // Temporary storage (max 100)
  byte GPSbytes = 0; // New char
  unsigned long GPStime = 0;
  
  dnt_count = 0;
  
  // analog (2), acc (3), mag (3), gyro (3), temp (1)
  int data[14];
  int32_t pressure = 0;

  // Data time stamp (in milliseconds)
  unsigned long time = 0; // Time of last/current sample

  while(1) {
    if ((micros() - GPStime) > GPS_INTERVAL) {
      GPSbytes = bobShield->gpsSendGet(GPSdata);
      if (GPSbytes > 0) {
        GPStime = micros();
        delayMicroseconds(500);
        
//        // Send data over the radio
        bobShield->dntSendData((uint8_t *)(&GPS_SENTINEL),1);
        bobShield->dntSendData((uint8_t *)(&GPStime),4);
        bobShield->dntSendData((uint8_t *)GPSdata,GPSbytes);
      
//        // Send data over the USB
//        Serial.write(GPS_SENTINEL);
//        Serial.write((byte *)(&GPStime),4);
//        Serial.write((byte *)GPSdata,24);
        
        // Send data to the nandflash
        nandFlash->writeBytes(&GPS_SENTINEL,1);
        nandFlash->writeBytes((byte *)(&GPStime),4);
        nandFlash->writeBytes((byte *)GPSdata,GPSbytes);
        
        // Don't overwhelm the DNT
        delayMicroseconds(10);
        time = micros();
        GPStime = time;
      }
    }

    // Control the sampling cadence
    if ((micros() - time) > SAMPLE_INTERVAL)
    {
      // Collect data as close to time stamp as possible
      time = micros();
      compass.read();
      gyro.read();
      data[0] = (unsigned int)analogRead(PRESS); //dummy read
      data[0] = (unsigned int)analogRead(PRESS); //dummy read
      data[0] = (unsigned int)analogRead(PRESS);
      pressure= ps.readPressureRaw();
      data[11]= (int)ps.readTemperatureRaw();
      data[2] = (int)compass.a.x;
      data[3] = (int)compass.a.y;
      data[4] = (int)compass.a.z;
      data[5] = (int)compass.m.x;
      data[6] = (int)compass.m.y;
      data[7] = (int)compass.m.z;
      data[8] = (int)gyro.g.x;
      data[9] = (int)gyro.g.y;
      data[10]= (int)gyro.g.z;
      data[1] = (unsigned int)analogRead(TEMP); //dummy read
      data[1] = (unsigned int)analogRead(TEMP); //dummy read
      data[1] = (unsigned int)analogRead(TEMP);

      dnt_count++;
      dnt_count = dnt_count % dnt_rate;

      // Send data over the radio
      if(!dnt_count) {
        bobShield->dntSendData((uint8_t *)(&DATA_SENTINEL),1);
        bobShield->dntSendData((uint8_t *)(&time),4);
        bobShield->dntSendData((uint8_t *)data,24);
        bobShield->dntSendData((uint8_t *)(&pressure),4);
      }

//      // Send data over the USB
//      Serial.write(DATA_SENTINEL);
//      Serial.write((byte *)(&time),4);
//      Serial.write((byte *)data,24);
//      Serial.write((byte *)(&pressure),4);

      // Send it to the nandflash
      nandFlash->writeBytes(&DATA_SENTINEL,1);
      nandFlash->writeBytes((byte *)(&time),4);
      nandFlash->writeBytes((byte *)data,24);
      nandFlash->writeBytes((uint8_t *)(&pressure),4);
    }
  }
}

//    if (MAX_GPS_DATA >= GPSindex)
//    {
//      // Send complete string (or at least as much as you have)
////        bobShield->dntSendData(GPSdata,GPSindex-1);
//      Serial.write(GPSdata,GPSindex-1);
//      nandFlash->writeBytes(GPSdata,GPSindex-1);
//      delayMicroseconds(10);
//
//      GPSdata[0] = GPS_SENTINEL;
//      long GPStime = micros(); // timestamp for the next string
//      GPSdata[1] = (char)(GPStime & 0xFF);
//      GPSdata[2] = (char)((GPStime >> 8) & 0xFF);
//      GPSdata[3] = (char)((GPStime >> 16) & 0xFF);
//      GPSdata[4] = (char)((GPStime >> 24) & 0xFF);
//
//      GPSindex = 5; // Reset index counter
//
//      time = micros(); // So the DNT doesn't get overrun
//    }
//    GPSchar = bobShield->gpsGet(&(GPSdata[GPSindex]),MAX_GPS_DATA-GPSindex);
//    GPSindex += GPSchar;
//    for (int i = GPSindex-GPSchar; i < GPSindex; i++) {
//      if (GPSdata[i] == GPS_SENTINEL) {
//        // Send complete string (or at least as much as you have)
////        bobShield->dntSendData(GPSdata,GPSindex-1);
//        Serial.write(GPSdata,GPSindex-1);
//        nandFlash->writeBytes(GPSdata,GPSindex-1);
//        delayMicroseconds(10);
//
//        GPSdata[0] = GPS_SENTINEL;
//        long GPStime = micros(); // timestamp for the next string
//        GPSdata[1] = (char)(GPStime & 0xFF);
//        GPSdata[2] = (char)((GPStime >> 8) & 0xFF);
//        GPSdata[3] = (char)((GPStime >> 16) & 0xFF);
//        GPSdata[4] = (char)((GPStime >> 24) & 0xFF);
//        // Don't lose data past next sentinel, assume only one more GPGGA line
//        for (int j = i; j < GPSindex; j++)
//          GPSdata[5+j-i] = GPSdata[j];
//        GPSindex = 5 + GPSindex - i; // Reset index counter
//
//        time = micros(); // So the DNT doesn't get overrun
//        break;
//      }
//    }


