
// Begin include libraries
#include <BobShield.h> // Custom library for Bob Shield v3
#include <Commands.h>
#include <GPGGA.h>

/* In "/libraries/Wire/utility/twi.h" make sure TWI_FREQ (line 28) is defined as 400000L */
//#include <Wire.h> // I2C communications (supporting the LSM303 library)
//#include <LSM303.h> // Library for the compass board
//#include <L3G.h> // Library for the L3GD20 gyro
//#include <LPS331.h> // Library for a pressure sensor

#include <Flash.h> // Library for the NAND Flash
#include <SPI.h> // SPI communications (supporting the NAND-Flash library)
#include <EEPROM.h> // For non-volatile memory on the Arduino (supporting the NAND-Flash library)
// End include libraries

#define SAMPLE_INTERVAL 10000 // 100 Hz
#define GPS_INTERVAL 100000 // 10 Hz
#define dnt_rate 3
#define MAX_GPS_DATA 100 // 100 bytes
#define TEMP A1
#define VOLTAGE_ARD A0
#define VOLTAGE_DNT A2

#define nandReset 0
#define ID 30

byte counter;
byte dnt_count;

Flash* nandFlash; // Pointer to nandflash object
BobShield* bobShield; // Pointer to BobShield object
//LSM303 compass; // Compass object
//L3G gyro; // Gyro object
//LPS331 ps; // Pressure sensor object

// At the start of each line of data
byte GPS_SENTINEL = '$'; // To indicate GPS data
byte DATA_SENTINEL = '#'; // To indicate a line of compass data

void setup() {

  Serial.begin(115200);
  nandFlash = new Flash(true, nandReset); // Flash(enable_write, restart_address_counter)
  bobShield = new BobShield();

  // Compass setup
//  Wire.begin();

//  ps.init(); //Turns on LPS331
//  ps.enableDefault();
//  ps.writeReg(LPS331_CTRL_REG1, 0x70); // ODR 25 Hz (decreases resolution)

  // adjust the prescale for the analog inputs (http://arduino.cc/forum/index.php/topic,6549.0.html)
  // defines for setting and clearing register bit
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

  // set prescale to 32
  sbi(ADCSRA,ADPS2) ;
  cbi(ADCSRA,ADPS1) ;
  sbi(ADCSRA,ADPS0) ;

//  pinMode(PRESS,INPUT);
  pinMode(TEMP,INPUT);
  pinMode(VOLTAGE_ARD,INPUT);
  pinMode(VOLTAGE_DNT,INPUT);
  
  bobShield->dntReset();
}

void loop() {
  // GPS data
  byte GPSdata[MAX_GPS_DATA]; // Temporary storage (max 100)
  byte GPSbytes = 0; // New char
  unsigned long GPStime = 0;
  
  counter = 0;
  dnt_count = 0;
  
  // temp(1),volt(2),identification(1),counter(1),empty(9)
  int data[14];

  // Data time stamp (in milliseconds)
  unsigned long time = 0; // Time of last/current sample

  while(1) {
    if ((micros() - GPStime) > GPS_INTERVAL) {
      GPSbytes = bobShield->gpsSendGet(GPSdata);
      if (GPSbytes > 0) {
        GPStime = micros();
        delayMicroseconds(500);
        
//        // Send data over the radio
        bobShield->dntSendData((uint8_t *)(&GPS_SENTINEL),1);
        bobShield->dntSendData((uint8_t *)(&GPStime),4);
        bobShield->dntSendData((uint8_t *)GPSdata,GPSbytes);
      
//        // Send data over the USB
//        Serial.write(GPS_SENTINEL);
//        Serial.write((byte *)(&GPStime),4);
//        Serial.write((byte *)GPSdata,24);
        
        // Send data to the nandflash
        nandFlash->writeBytes(&GPS_SENTINEL,1);
        nandFlash->writeBytes((byte *)(&GPStime),4);
        nandFlash->writeBytes((byte *)GPSdata,GPSbytes);
        
        // Don't overwhelm the DNT
        delayMicroseconds(10);
        time = micros();
        GPStime = time;
      }
    }

    // Control the sampling cadence
    if ((micros() - time) > SAMPLE_INTERVAL)
    {
      // Collect data as close to time stamp as possible
      time = micros();
      
        data[0] = (unsigned int)analogRead(VOLTAGE_ARD);  //dummy read
        data[0] = (unsigned int)analogRead(VOLTAGE_ARD);  //dummy read
        data[0] = (unsigned int)analogRead(VOLTAGE_ARD);
        data[1] = (unsigned int)analogRead(TEMP);  //dummy read
        data[1] = (unsigned int)analogRead(TEMP);  //dummy read
        data[1] = (unsigned int)analogRead(TEMP);
        data[2] = (unsigned int)analogRead(VOLTAGE_DNT);  //dummy read
        data[2] = (unsigned int)analogRead(VOLTAGE_DNT);  //dummy read
        data[2] = (unsigned int)analogRead(VOLTAGE_DNT);
        data[3] = ID;  
        data[4] = counter;
        data[5] = 1;      //empty data spots so the array is still size 14
        data[6] = 1;
        data[7] = 1;
        data[8] = 1;
        data[9] = 1;
        data[10] = 1;
        data[11] = 1;
        
        counter++;
        dnt_count++;
        dnt_count = dnt_count % dnt_rate;

      // Send data over the radio
      if (!dnt_count) {
        bobShield->dntSendData((uint8_t *)(&DATA_SENTINEL),1);
        bobShield->dntSendData((uint8_t *)(&time),4);
        bobShield->dntSendData((uint8_t *)data,24);
      }

//      // Send data over the USB
//      Serial.write(DATA_SENTINEL);
//      Serial.write((byte *)(&time),4);
//      Serial.write((byte *)data,24);

      // Send it to the nandflash
      nandFlash->writeBytes(&DATA_SENTINEL,1);
      nandFlash->writeBytes((byte *)(&time),4);
      nandFlash->writeBytes((byte *)data,24);
    }
  }
}
